from pelican.utils import slugify, truncate_html_words


class Page(object):
    """Represents a page..
    Given a content, and metadatas, create an adequate object.

    :param string: the string to parse, containing the original content.
    :param markup: the markup language to use while parsing.
    """
    mandatory_properties = ('title',)

    def __init__(self, content, metadatas={}, settings={}, filename=None):
        self.content = content
        for key, value in metadatas.items():
            setattr(self, key, value)

        if not hasattr(self, 'author'):
            if 'AUTHOR' in settings:
                self.author = settings['AUTHOR']

        if filename:
            self.filename = filename

    def check_properties(self):
        """test that each mandatory property is set."""
        for prop in self.mandatory_properties:
            if not hasattr(self, prop):
                raise NameError(prop)

    @property
    def url(self):
        return '%s.html' % self.slug

    @property
    def slug(self):
        return slugify(self.title)

    @property
    def summary(self):
        return truncate_html_words(self.content, 50)


class Article(Page):
    mandatory_properties = ('title', 'date', 'category')


class Quote(Page):
    base_properties = ('author', 'date')


def is_valid_content(content, f):
    try:
        content.check_properties()
        return True
    except NameError as e:
        print u" [info] Skipping %s: impossible to find informations about '%s'" % (f, e)
        return False



from operator import attrgetter
import os

from rst2pdf.createpdf import RstToPdf

from pelican.utils import update_dict, copytree
from pelican.contents import Article, Page, is_valid_content
from pelican.readers import read_file

_DIRECT_TEMPLATES = ('index', 'tags', 'categories', 'archives')


class Processor(object):

    def _update_context(self, context, items):
        """Update the context with the given items from the currrent 
        processor.
        """
        for item in items:
            value = getattr(self, item)
            if hasattr(value, 'items'):
                value = value.items()
            context[item] = value


class ArticlesProcessor(Processor):

    def __init__(self, settings=None):
        self.articles = [] 
        self.dates = {}
        self.years = {}
        self.tags = {}
        self.categories = {} 
        
    def generate_feeds(self, context, generator):
        """Generate the feeds from the current context, and output files.""" 

        generator.generate_feed(self.articles, context, context['FEED'])

        for cat, arts in self.categories.items():
            arts.sort(key=attrgetter('date'), reverse=True)
            generator.generate_feed(arts, context, 
                                    context['CATEGORY_FEED'] % cat)

    def generate_pages(self, context, generator):
        """Generate the pages on the disk"""

        templates = generator.get_templates()
        generate = generator.generate_file
        for template in _DIRECT_TEMPLATES:
            generate('%s.html' % template, templates[template], context, blog=True)
        for tag in self.tags:
            generate('tag/%s.html' % tag, templates['tag'], context, tag=tag)
        for cat in self.categories:
            generate('category/%s.html' % cat, templates['category'], context,
                          category=cat, articles=self.categories[cat])
        for article in self.articles:
            generate('%s' % article.url,
                          templates['article'], context, article=article,
                          category=article.category)

    def preprocess(self, context, generator):

        # build the list of articles / categories / etc.
        files = generator.get_files(generator.path, exclude=['pages',])
        for f in files:
            content, metadatas = read_file(f)
            if 'category' not in metadatas.keys():
                category = os.path.dirname(f).replace(
                    os.path.expanduser(generator.path)+'/', '')

                if category != '':
                    metadatas['category'] = unicode(category)

            article = Article(content, metadatas, settings=generator.settings,
                              filename=f)
            if not is_valid_content(article, f):
                continue

            update_dict(self.dates, article.date.strftime('%Y-%m-%d'), article)
            update_dict(self.years, article.date.year, article)
            update_dict(self.categories, article.category, article)
            if hasattr(article, 'tags'):
                for tag in article.tags:
                    update_dict(self.tags, tag, article)
            self.articles.append(article)

        # sort the articles by date
        self.articles.sort(key=attrgetter('date'), reverse=True)
        # and generate the output :)
        self._update_context(context, ('articles', 'dates', 'years', 
                                       'tags', 'categories'))

    def process(self, context, generator):
        self.generate_feeds(context, generator)
        self.generate_pages(context, generator)


class PagesProcessor(Processor):
    """Generate pages"""

    def __init__(self):
        self.pages = []

    def preprocess(self, context, generator):
        for f in generator.get_files(os.sep.join((generator.path, 'pages'))):
            content, metadatas = read_file(f)
            page = Page(content, metadatas, settings=generator.settings,
                        filename=f)
            if not is_valid_content(page, f):
                continue
            self.pages.append(page)

        context['PAGES'] = self.pages
        
    def process(self, context, generator):
        templates = generator.get_templates()
        for page in self.pages:
            generator.generate_file('pages/%s' % page.url, 
                               templates['page'], context, page=page)
        self._update_context(context, ('pages',))


class StaticProcessor(Processor):
    """copy static paths to output"""

    def process(self, context, generator):
        for path in generator.settings['STATIC_PATHS']:
            copytree(path, generator.theme, generator.output_path)
        copytree('pics', generator.path, generator.output_path)


class PdfProcessor(Processor):
    """Generate PDFs on the output dir, for all articles and pages coming from
    rst"""

    def _create_pdf(self, creator, obj, output_path):
        if obj.filename.endswith(".rst"):
            creator.createPdf(text=open(obj.filename).read(),
                output=os.path.join(output_path, "%s.pdf" % obj.slug))

    def process(self, context, generator):
        pdf_path = os.path.join(generator.output_path, 'pdf')
        try:
            os.mkdir(pdf_path)
        except OSError:
            pass

        pdfcreator = RstToPdf(breakside=0, stylesheets=['twelvepoint'])

        for article in context['articles']:
            self._create_pdf(pdfcreator, article, pdf_path)

        for page in context['pages']:
            self._create_pdf(pdfcreator, page, pdf_path)



# -*- coding: utf-8 -*-
import os
from codecs import open

from jinja2 import Environment, FileSystemLoader
from jinja2.exceptions import TemplateNotFound
from feedgenerator import Atom1Feed

from pelican.settings import read_settings

_TEMPLATES = ('index', 'tag', 'tags', 'article', 'category', 'categories',
              'archives', 'page')


class Generator(object):
    """Base class generator"""
    
    def __init__(self, settings=None, path=None, theme=None, output_path=None, 
        markup=None):
        if settings is None:
            settings = {}
        self.settings = read_settings(settings)
        self.path = path or self.settings['PATH']
        self.theme = theme or self.settings['THEME']
        output_path = output_path or self.settings['OUTPUT_PATH']
        self.output_path = os.path.realpath(output_path)
        self.markup = markup or self.settings['MARKUP']

        if 'SITEURL' not in self.settings:
            self.settings['SITEURL'] = self.output_path

        # get the list of files to parse
        if not path:
            raise Exception('you need to specify a path to search the docs on !')

    def run(self, processors):
        context = self.settings.copy()
        processors = [p() for p in processors]

        for p in processors:
            if hasattr(p, 'preprocess'):
                p.preprocess(context, self)

        for p in processors:
            p.process(context, self)

    def generate_feed(self, elements, context, filename=None):
        """Generate a feed with the list of articles provided

        Return the feed. If no output_path or filename is specified, just return
        the feed object.

        :param articles: the articles to put on the feed.
        :param context: the context to get the feed metadatas.
        :param output_path: where to output the file.
        :param filename: the filename to output.
        """
        feed = Atom1Feed(
            title=context['SITENAME'],
            link=context['SITEURL'],
            feed_url='%s/%s' % (context['SITEURL'], filename),
            description=context.get('SITESUBTITLE', ''))
        for element in elements:
            feed.add_item(
                title=element.title,
                link='%s/%s' % (context['SITEURL'], element.url),
                description=element.content,
                author_name=getattr(element, 'author', 'John Doe'),
                pubdate=element.date)

        if filename:
            complete_path = os.path.join(self.output_path, filename)
            try:
                os.makedirs(os.path.dirname(complete_path))
            except Exception:
                pass
            fp = open(complete_path, 'w')
            feed.write(fp, 'utf-8')
            print u' [ok] writing %s' % complete_path
            
            fp.close()
        return feed

    def generate_file(self, name, template, context, **kwargs):
        """Write the file with the given informations

        :param name: name of the file to output
        :param template: template to use to generate the content
        :param context: dict to pass to the templates.
        :param **kwargs: additional variables to pass to the templates
        """
        context = context.copy()
        context.update(kwargs)
        output = template.render(context)
        filename = os.sep.join((self.output_path, name))
        try:
            os.makedirs(os.path.dirname(filename))
        except Exception:
            pass
        with open(filename, 'w', encoding='utf-8') as f:
            f.write(output)
        print u' [ok] writing %s' % filename

    def get_templates(self):
        """Return the templates to use.

        :param path: the path to load the templates from
        """
        path = os.path.expanduser(os.path.join(self.theme, 'templates'))
        env = Environment(loader=FileSystemLoader(path))
        templates = {}
        for template in _TEMPLATES:
            try:
                templates[template] = env.get_template('%s.html' % template)
            except TemplateNotFound:
                raise Exception('Unable to load %s.html from %s' % (
                    template, path))
        return templates

    def get_files(self, path, exclude=[]):
        """Return the files to use to use in this generator

        :param path: the path to search the file on
        :param exclude: the list of path to exclude
        """
        files = []
        for root, dirs, temp_files in os.walk(path, followlinks=True):
            for e in exclude:
                if e in dirs:
                    dirs.remove(e)
            files.extend([os.sep.join((root, f)) for f in temp_files
                if True in [f.endswith(markup) for markup in self.markup]])
        return files
